import { Inject, Provide } from '@midwayjs/core';
import { FrameworkErrorEnum, GetModel, ServiceError } from 'free-midway-component';
import { get, isEqual, upperFirst } from 'lodash';
import { LoginDTO, LoginType, UPwdDTO } from '../interface/login.interface';
import { RoleService } from '../service/system/role.service';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import { User } from '../entity/system/user.entity';

@Provide()
export class LoginService {
  @Inject()
  roleService: RoleService;

  @InjectEntityModel(User)
  uModel: ReturnModelType<typeof User>;

  /**
   * 账密登录
   * @param data 用户名和密码
   * @param type 用户类型
   * @returns 用户信息/空值
   */
  async loginByAccount(data: LoginDTO, type: LoginType) {
    const model = GetModel(upperFirst(type));
    const user = await model.findOne({ account: data.account }, '+password').lean();
    if (!user) throw new ServiceError('未找到用户信息', FrameworkErrorEnum.NOT_FOUND_DATA);
    await this.checkAccountCanLogin(user, type);
    if (!isEqual(user.password.secret, data.password)) throw new ServiceError('密码错误', FrameworkErrorEnum.SERVICE_FAULT);
    return user;
  }
  /**
   * 检查用户是否可以登录(从用户本身和角色检查)
   * @param user 用户信息
   * @param type 用户类型
   */
  async checkAccountCanLogin(user, type: LoginType) {
    // 判断是否可以使用:
    // 管理员: 超级管理员无视,直接往下; 普通管理员需要查看is_use是不是'0'
    // 其他用户需要看status是不是'1';
    if (type === 'Admin') {
      if (get(user, 'is_super') === '1') {
        if (get(user, 'is_use') === '1') throw new ServiceError('该用户已被禁用', FrameworkErrorEnum.SERVICE_FAULT);
      }
    } else {
      if (get(user, 'status') !== '1') throw new ServiceError('该用户已被禁用', FrameworkErrorEnum.SERVICE_FAULT);
      const role = await this.roleService.findOne({ code: type, is_use: '0' });
      if (!role) throw new ServiceError('当前角色下的用户无法使用!', FrameworkErrorEnum.SERVICE_FAULT);
    }
  }

  async updatePwd(data: UPwdDTO, type: LoginType) {
    const model = GetModel(upperFirst(type));
    const user = await model.findById(data._id);
    if (!user) new ServiceError('未找到用户信息!', FrameworkErrorEnum.DATA_NOT_FOUND);
    user.password = data.password;
    await user.save();
  }
  // openid登录
  async wxAppLogin(openid: string) {
    const user = await this.uModel.findOne({ openid }).lean();
    return user
  }
}
